# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.manager
import blogging.models
import django.contrib.sites.managers


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        ('blogging', '0002_auto_20150306_1840'),
    ]

    operations = [
        migrations.CreateModel(
            name='Picture',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Nom')),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('description', models.TextField(verbose_name='Description', blank=True)),
                ('image', models.ImageField(max_length=200, upload_to=blogging.models.upload_to_blogging)),
                ('site', models.ForeignKey(default=1, verbose_name='Site', to='sites.Site')),
            ],
        ),
        migrations.AlterModelManagers(
            name='category',
            managers=[
                (b'objects', django.db.models.manager.Manager()),
                (b'on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
        migrations.AlterModelManagers(
            name='post',
            managers=[
                (b'objects', django.db.models.manager.Manager()),
                (b'on_site', django.contrib.sites.managers.CurrentSiteManager()),
            ],
        ),
    ]



import datetime

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.utils import override_settings
from django.test import Client
from django.core.urlresolvers import reverse
from django.conf import settings

from blogging.models import Post


class PostUrlsTestCase(TestCase):
    def setUp(self):
        self.author = User.objects.create(username='test_user')
        self.client = Client()

    def test_not_allow_empty_list(self):
        # no post, raise 404
        response = self.client.get(reverse('blog-index'))
        self.assertEqual(response.status_code, 404)

        post_1, created = Post.objects.get_or_create(
            title="post 1", slug="post-1",
            published_on=datetime.datetime(2010, 1, 1),
            status=Post.PUBLISHED, author=self.author
        )
        # created one published post, HTTP 200
        response = self.client.get(reverse('blog-index'))
        self.assertEqual(response.status_code, 200)



#-*- coding: utf-8 -*-
from django.views.generic import ListView
from django.views.generic import DetailView
from django.views.generic import TemplateView

from django.shortcuts import get_object_or_404

from blogging.models import Post, Category
from blogging.settings import conf


class PostListView(ListView):
    """
    Display a list of the item for the current user (for one feed or for all
    the feeds it is subscribed).

    Context contains :
    'current_category': the current category if available (None if not)
    'object_list': the list of the items to display
    """

    allow_empty = conf['ALLOW_EMPTY']
    paginate_by = conf['ITEMS_BY_PAGE']

    def get_queryset(self):
        """
        Return the available posts (filtered by category if needed)
        """
        if 'category_slug' in self.kwargs:
            self.category = get_object_or_404(
                Category.availables,
                slug=self.kwargs['category_slug']
            )
            return Post.availables.published().filter(categories=self.category)
        else:
            self.category = None
            return Post.availables.published()

    def get_context_data(self, **kwargs):
        """
        Add current category to the context
        """
        context = super(PostListView, self).get_context_data(**kwargs)
        context['current_category'] = self.category
        return context


class PostDetailView(DetailView):
    """
    Display / Preview a particular item
    """

    def get_queryset(self):
        if self.kwargs.get('preview', False) and self.request.user.is_staff:
            queryset = Post.on_site.all()
        else:
            queryset = Post.availables.published()
        return queryset


class ArchivesDetailsListView(ListView):
    """
    Display the archive for a particular month
    """
    template_name = 'blogging/post_list.html'
    paginate_by = conf['ITEMS_BY_PAGE']

    def get_queryset(self):
        items = Post.availables.published()
        items = items.filter(
            published_on__year=int(self.kwargs['year']),
            published_on__month=int(self.kwargs['month'])
        )
        return items


class ArchivesView(TemplateView):
    template_name = "blogging/archives.html"

    def get_context_data(self, **kwargs):
        context = super(ArchivesView, self).get_context_data(**kwargs)
        context['object_list'] = Post.availables.published()
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blogging', '0003_auto_20150504_1809'),
    ]

    operations = [
        migrations.AddField(
            model_name='category',
            name='picture',
            field=models.ForeignKey(verbose_name='Picture', blank=True, to='blogging.Picture', null=True),
        ),
        migrations.AddField(
            model_name='post',
            name='main_picture',
            field=models.ForeignKey(verbose_name='Picture', blank=True, to='blogging.Picture', null=True),
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib.sitemaps import Sitemap
from blogging.models import Post


class BlogSitemap(Sitemap):
    priority = 0.5

    def items(self):
        return Post.availables.published()

    def lastmod(self, obj):
        return obj.updated_on



# -*- coding: utf-8 -*-
from blogging.models import Category, Post


def categories(request):
    """
    Return the site categories
    """
    categories = Category.availables.all()
    return {'categories': categories}


def latest_posts(request):
    """
    Return the latest posts
    """
    latest_items = Post.availables.published()[:5]
    return {'latest_posts': latest_items}


def month_with_items(request):
    """
    Return the latest posts
    """
    months = Post.availables.published().dates('published_on', 'month')
    return {'month_with_items': months}


def selected_items(request):
    """
    Return a list of selected items
    """
    selected_items = Post.availables.published().filter(selected=True)
    return {'selected_items': selected_items}



# -*- coding: utf-8 -*-

import re

from django import template
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.conf import settings

register = template.Library()

from blogging.models import Post

class LatestPostNode(template.Node):
    def __init__(self, number_var, var_name, options):
        self.var_name = var_name
        self.number_var = template.Variable(number_var)
        self.options = options

    def render(self, context):
        try:
            number_var = int(self.number_var.resolve(context))
        except template.VariableDoesNotExist:
            number_var = 10
        queryset = Post.availables.published()
        if 'networks' in self.options:
            queryset = Post.objects.published()
            queryset = queryset.exclude(site__id__exact=settings.SITE_ID)
        context[self.var_name] = queryset[:number_var]
        return ''


@register.tag(name="latest_posts")
def latest_posts(parser, token):
    """
    {% latest_posts 3 as latest_posts [networks] %}
    """
    tokens = token.split_contents()
    number_var = tokens[1]
    var_name = tokens[3]
    try:
        options = tokens[4].split(",")
    except IndexError:
        options = []
    return LatestPostNode(number_var, var_name, options)



# -*- coding: utf-8 -*-
import re

from django import template
from django.conf import settings
from django.core.cache import cache
from django.core.urlresolvers import reverse
from django.template.loader import render_to_string

register = template.Library()


class DisqusCommentNode(template.Node):
    def __init__(self, kwargs):
        self.kwargs = kwargs
        self.template_name = "blogging/partials/disqus.html"

    def render(self, context):
        try:
            disqus_shortname = settings.BLOGGING_CONFIG['DISQUS_SHORTNAME']
        except (AttributeError, KeyError):
            raise Exception("Please provide a settings.BLOGGING_CONFIG['DISQUS_SHORTNAME'] attribute")
        template_context = {
            'disqus_shortname': disqus_shortname
        }
        for kwarg in self.kwargs:
            args_elements = kwarg.split("=")
            if len(args_elements) == 2:
                var_name, var_value = args_elements
                has_double_quote = (
                    var_value.startswith("'") and var_value.endswith("'")
                )
                has_simple_quote = (
                    var_value.startswith('"') and var_value.endswith('"')
                )
                if has_double_quote or has_simple_quote:
                    var_value = var_value[1:-1]
                else:
                    var = template.Variable(var_value)
                    var_value = var.resolve(context)
                template_context[var_name] = var_value
            else:
                raise Exception("error in disqus_comments")
        rendered = render_to_string(
            "blogging/partials/disqus.html",
            template_context
        )
        
        return rendered


@register.tag(name="disqus_comments")
def disqus_comments(parser, token):
    """
    {% disqus_comments shortname="username" %}
    """
    tokens = token.split_contents()
    kwargs = tokens[1:]
    return DisqusCommentNode(kwargs)



# -*- coding: utf-8 -*-
from django.utils.translation import ugettext as _
from django.contrib.syndication.views import Feed
from django.contrib.sites.models import Site
from django.shortcuts import get_object_or_404
from django.conf import settings

from blogging.models import Post, Category

# TODO: move it to the new blogging conf
description_template = getattr(settings, 'BLOGGING_FEED_DESCRIPTION_TEMPLATE', "blogging/feeds/description.html")
title_template = getattr(settings, 'BLOGGING_FEED_TITLE_TEMPLATE', "blogging/feeds/title.html")
feed_title = getattr(settings, 'BLOGGING_FEED_TITLE', None)

class LatestEntriesByCategory(Feed):
    description_template = description_template
    title_template = title_template
    
    def get_object(self, request, category_slug):
        site = Site.objects.get_current()
        return get_object_or_404(Category, slug=category_slug, site=site)

    def title(self, category):
        return category.name

    def link(self, obj):
        return obj.get_absolute_url()

    def description(self, category):
        return _(u"Latest published items from %s") % category

    def items(self, category):
        return Post.availables.published().filter(categories=category)[:20]

class LatestEntries(Feed):
    description_template = description_template
    title_template = title_template
    link = "/"
    description = _(u"Latest published entries")

    def title(self):
        return feed_title or _(u"Latest entries")

    def items(self):
        return Post.availables.published()[:20]
    
    def item_pubdate(self, item):
        return item.published_on
    
    def item_author_name(self, item):
        return item.author.get_full_name()
    
    def item_categories(self, item):
        return item.categories.all()
    



from django.conf.urls import url


from blogging.views import PostDetailView
from blogging.urls.base import base_patterns

urlpatterns = []
urlpatterns += base_patterns
urlpatterns += [
    url(r'^(?P<slug>[-\w]+)$', PostDetailView.as_view(), name="blog-item"),
]



# -*- coding: utf-8 -*-
VERSION = (1, 2, 6, 'final', 0)


def get_version():
    """Get app version."""
    version = "{0}.{1}".format(VERSION[0], VERSION[1])
    if VERSION[2]:
        version = "{0}.{1}".format(version, VERSION[2])
    if VERSION[3:] == ('alpha', 0):
        version = "{0} pre-alpha".format(version)
    else:
        if VERSION[3] != 'final':
            version = "{0} {1} {2}".format(version, VERSION[3], VERSION[4])
    return version



"""
The goal of this module is to define the default values for the blog settings.
It try to get the variable value from the main settings.py
"""
from django.conf import settings

conf = {
    'BLOG_ITEM_URL': 'long',
    'ITEMS_BY_PAGE': 10,
    'ALLOW_EMPTY': False
}
# Check if settings has a DEBUG_TOOLBAR_CONFIG and updated config
conf.update(getattr(settings, 'BLOGGING_CONFIG', {}))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('blogging', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='post',
            name='trackback_open',
        ),
        migrations.AddField(
            model_name='category',
            name='all_posts_count',
            field=models.IntegerField(default=0, verbose_name='All posts in category', editable=False),
        ),
        migrations.AddField(
            model_name='category',
            name='visible_posts_count',
            field=models.IntegerField(default=0, verbose_name='Visible posts in category', editable=False),
        ),
        migrations.AlterField(
            model_name='post',
            name='published_on',
            field=models.DateTimeField(verbose_name='Publi\xe9 le', db_index=True),
        ),
    ]



#-*- coding: utf-8 -*-


def tokenize(*args):
    """
    Return the tokens for the string passed as parameters
    """
    raw_keywords = []
    for arg in args:
        raw_keywords += arg.split()
    cleaned_keywords = []
    for keyword in raw_keywords:
        keyword = keyword.strip("(")
        keyword = keyword.strip(")")
        keyword = keyword.strip(",")
        keyword = keyword.strip(".")
        cleaned_keywords.append(keyword)



# -*- coding: utf-8 -*-
from django.contrib import admin

from blogging.actions import make_draft
from blogging.actions import make_published
from blogging.actions import make_selected
from blogging.actions import update_counters
from blogging.models import Category
from blogging.models import Post
from blogging.models import Picture


class PictureAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("name",)}
    list_display = (
        'name', 'site', 'description', 'image',
    )
    list_filter = ('site',)
    search_fields = ('name', 'description')


class CategoryAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("name",)}
    list_display = (
        'name', 'site', 'visible_posts_count', 'all_posts_count',
    )
    list_filter = ('site',)
    search_fields = ('name',)
    actions = [update_counters,]


class PostAdmin(admin.ModelAdmin):
    list_display = (
        'title', 'author', 'status', 'published_on', 'selected', 'site'
    )
    list_filter = [
        'site', 'author', 'status', 'selected', 'categories'
    ]
    date_hierarchy = 'published_on'
    prepopulated_fields = {"slug": ("title",)}
    search_fields = ('excerpt', 'content', 'item__title')
    actions = [make_published, make_draft, make_selected]
    filter_horizontal = ["categories"]

    def formfield_for_manytomany(self, db_field, request, **kwargs):
        """
        Monkey patching the form field for categories
        TODO: Create a widget to manage it more easily
        """
        field = super(PostAdmin, self).formfield_for_manytomany(db_field, request, **kwargs)
        field.queryset = field.queryset.order_by('site__domain')
        field.label_from_instance = lambda obj: "%(site)s - %(name)s" % {
            'site': obj.site, 'name': obj.name
        }
        return field

admin.site.register(Picture, PictureAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Post, PostAdmin)



# -*- coding: utf-8 -*-
from django.utils.translation import ungettext
from django.utils.translation import ugettext_lazy as _

from blogging.models import Post


# Post Actions
def make_published(modeladmin, request, queryset):
    """
    Mark the given posts as published
    """
    count = queryset.update(status=Post.PUBLISHED)
    message = ungettext(
        u'%(count)d post was successfully marked as published.',
        u'%(count)d posts were successfully marked as published',
        count
    ) % {'count': count}
    modeladmin.message_user(request, message)
make_published.short_description = _(u"Mark selected stories as published")


def make_draft(modeladmin, request, queryset):
    """
    Mark the given posts as draft
    """
    count = queryset.update(status=Post.DRAFT)
    message = ungettext(
        u'%(count)d post was successfully marked as draft.',
        u'%(count)d posts were successfully marked as draft',
        count
    ) % {'count': count}
    modeladmin.message_user(request, message)
make_draft.short_description = _(u"Mark selected stories as draft")


def make_selected(modeladmin, request, queryset):
    """
    Mark the given posts as selected
    """
    count = queryset.update(selected=True)
    message = ungettext(
        u'%(count)d post was successfully marked as selected.',
        u'%(count)d posts were successfully marked as selected',
        count
    ) % {'count': count}
    modeladmin.message_user(request, message)
make_selected.short_description = _(u"Mark selected stories as selected")


# Category Actions
def update_counters(modeladmin, request, queryset):
    """
    Update the counters for the given categories
    """
    count = 0
    for category in queryset:
        category.update_counters()
        count += 1
    message = ungettext(
        u'%(count)d category has been updated.',
        u'%(count)d categories had been updated.',
        count
    ) % {'count': count}
    modeladmin.message_user(request, message)
update_counters.short_description = _(u"Update categories counters")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.db.models.manager
from django.conf import settings
import django.contrib.sites.managers


class Migration(migrations.Migration):

    dependencies = [
        ('sites', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Nom')),
                ('slug', models.SlugField(verbose_name='Slug')),
                ('description', models.TextField(verbose_name='Description', blank=True)),
                ('site', models.ForeignKey(default=1, verbose_name='Site', to='sites.Site')),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'Cat\xe9gorie',
                'verbose_name_plural': 'Cat\xe9gories',
            },
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=150, verbose_name='Titre')),
                ('slug', models.SlugField(unique=True, max_length=150, verbose_name='Slug')),
                ('excerpt', models.TextField(verbose_name='Excerpt', db_column='exceprt', blank=True)),
                ('content', models.TextField(verbose_name='Contenu')),
                ('published_on', models.DateTimeField(verbose_name='Publi\xe9 le')),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('updated_on', models.DateTimeField(auto_now=True)),
                ('status', models.IntegerField(default=1, db_index=True, verbose_name='Status', choices=[(1, 'Brouillon'), (2, 'Publi\xe9'), (3, 'Supprim\xe9')])),
                ('selected', models.BooleanField(default=False, verbose_name='Selectionn\xe9')),
                ('comments_open', models.BooleanField(default=True, verbose_name='Commentaires ouverts ?')),
                ('trackback_open', models.BooleanField(default=False, verbose_name='Trackbacks ouverts ?')),
                ('author', models.ForeignKey(verbose_name='Auteur', to=settings.AUTH_USER_MODEL)),
                ('categories', models.ManyToManyField(to='blogging.Category', verbose_name='Cat\xe9gories')),
                ('site', models.ForeignKey(default=1, verbose_name='Site', to='sites.Site')),
            ],
            options={
                'ordering': ['-published_on'],
                'verbose_name': 'billet',
                'verbose_name_plural': 'billets',
            },
        ),
    ]



# -*- coding: utf-8 -*-

import re

from django import template
from django.core.cache import cache
from django.core.urlresolvers import reverse

register = template.Library()

from blogging.models import Category

class CategoriesNode(template.Node):
    def __init__(self, var_name):
        self.var_name = var_name

    def render(self, context):
        context[self.var_name] = Category.on_site.all().order_by('name')
        return ''


@register.tag(name="categories")
def categories(parser, token):
    """
    {% categories as categories %}
    """
    tokens = token.split_contents()
    if len(tokens) is not 3 and token[0] == 'categories' and token[0] == 'as':
        raise template.TemplateSyntaxError("%r tag must be used with %s" % (tokens[0], "{% categories as categories %}"))
    var_name = tokens[2]
    return CategoriesNode(var_name)

@register.filter
def startswith(value, arg):
    "Removes all values of arg from the given string"
    return value.startswith(arg)


from django.conf.urls import url

from blogging.views import PostListView, PostDetailView, ArchivesView, ArchivesDetailsListView

base_patterns = [
    url(r'^$', PostListView.as_view(), name='blog-index'),
    url(r'^archives/$', ArchivesView.as_view(), name='blog-archives'),
    url(r'^archives/(?P<year>\d{4})/(?P<month>\d{2})/$', ArchivesDetailsListView.as_view(), name='blog-archives-month'),
    url(r'^page-(?P<page>\d+)/$', PostListView.as_view(), name='blog-page'),
    url(r'^preview/(?P<slug>[-\w]+)/$', PostDetailView.as_view(), {'preview': True}, name='blog-preview-item'),
    url(r'^(?P<category_slug>[-\w]+)/$', PostListView.as_view(), name='blog-category'),
    url(r'^(?P<category_slug>[-\w]+)/page-(?P<page>\d+)/$', PostListView.as_view(), name='blog-category-page'),
]



# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from django.utils import timezone


# Category related
class AvailableCategoriesManager(models.Manager):
    def get_queryset(self):
        queryset = super(AvailableCategoriesManager, self).get_queryset()
        queryset = queryset.filter(site__id=settings.SITE_ID)
        return queryset

    def get_by_natural_key(self, slug, site_id):
        return self.get(slug=slug, site__id=site_id)


# Post related
class PostManager(models.Manager):
    def published(self):
        now = timezone.now()
        queryset = self.filter(published_on__lte=now)
        queryset = queryset.filter(status=self.model.PUBLISHED)
        return queryset

    def related_items(self, item, length=5):
        """
        Return items related to the item passed as parameter
        """
        qs = self.availables.published()
        qs = qs.filter(categories__in=item.categories.all())
        qs = qs.exclude(id=item.id)
        qs = qs.distinct()[:length]


class AvailableItemsManager(PostManager):
    def get_queryset(self):
        queryset = super(AvailableItemsManager, self).get_queryset()
        queryset = queryset.filter(site__id__exact=settings.SITE_ID)
        return queryset

    def get_by_natural_key(self, slug, site_id):
        return self.get(slug=slug, site__id=site_id)



from setuptools import setup, find_packages
import os

package = "django-blogging"
version = __import__('blogging').get_version().replace(' ', '-')


def read_file(filename):
    """Read a file into a string"""
    path = os.path.abspath(os.path.dirname(__file__))
    filepath = os.path.join(path, filename)
    try:
        return open(filepath).read()
    except IOError:
        return ''


def get_readme():
    """Return the README file contents. Supports text,rst, and markdown"""
    for name in ('README', 'README.rst', 'README.md'):
        if os.path.exists(name):
            return read_file(name)
    return ''

setup(
    name=package,
    version=version,
    author="Fabien Schwob",
    author_email="fabien@x-phuture.com",
    description="""
    Django blogging app. The goal is to keep it simple and configurable
    """.strip(),
    long_description=get_readme(),
    license="BSD",
    url="http://github.com/jibaku/blogging",
    packages=find_packages(exclude=[]),
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Framework :: Django",
    ]
)



# -*- coding: utf-8 -*-
import logging
import hashlib
import time

from django.conf import settings
from django.contrib.auth.models import User
from django.contrib.sites.managers import CurrentSiteManager
from django.contrib.sites.models import Site
from django.core.urlresolvers import reverse, NoReverseMatch
from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext as _
import django.dispatch

from blogging.managers import AvailableCategoriesManager
from blogging.managers import AvailableItemsManager
from blogging.managers import PostManager


def upload_to_blogging(instance, filename):
    hasher = hashlib.md5()
    hasher.update(filename.encode('utf-8'))
    hasher.update(str(time.time()))
    hashed_name = hasher.hexdigest()
    extension = filename.split('.')[-1]
    return "blogging/pictures/{0}-{1}.{2}".format(
        hashed_name[:5], instance.slug, extension
    )


class Picture(models.Model):
    name = models.CharField(_(u"Name"), max_length=100)
    slug = models.SlugField(_(u"Slug"))
    description = models.TextField(_(u"Description"), blank=True)
    site = models.ForeignKey(Site, verbose_name=_("Site"), default=settings.SITE_ID)
    image = models.ImageField(upload_to=upload_to_blogging, max_length=200)

    def __unicode__(self):
        return self.name


class Category(models.Model):
    """
    A category to regroup similar articles
    """
    name = models.CharField(_(u"Name"), max_length=100)
    slug = models.SlugField(_(u"Slug"))
    description = models.TextField(_(u"Description"), blank=True)
    site = models.ForeignKey(Site, verbose_name=_("Site"), default=settings.SITE_ID)
    picture = models.ForeignKey(Picture, verbose_name=_("Picture"), blank=True, null=True)

    # hidden cached field
    visible_posts_count = models.IntegerField(_(u"Visible posts in category"), editable=False, default=0)
    all_posts_count = models.IntegerField(_(u"All posts in category"), editable=False, default=0)

    objects = models.Manager()  # The default manager.
    on_site = CurrentSiteManager()
    availables = AvailableCategoriesManager()  # The Online manager.

    class Meta:
        ordering = ['name']
        verbose_name = _(u"category")
        verbose_name_plural = _(u"categories")

    def __unicode__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('blog-category', args=[self.slug])

    def natural_key(self):
        return [self.slug, self.site.id]

    def is_empty(self):
        return not Post.availables.published().filter(categories=self).exists()

    @property
    def get_online_posts_count(self):
        return Post.availables.published().filter(categories=self).count()

    @property
    def get_all_posts_count(self):
        return Post.availables.published().filter(categories=self).count()

    def update_counters(self):
        self.visible_posts_count = self.get_online_posts_count
        self.all_posts_count = self.get_all_posts_count
        self.save()


class Post(models.Model):
    """
    The Post contains the generic fields for a blog item, like the publication
    date, the author, the slug, etc.
    """
    # Constants for the blog status
    DRAFT = 1
    PUBLISHED = 2
    DELETED = 3
    STATUS_CHOICES = (
        (DRAFT, _(u'Draft')),
        (PUBLISHED, _(u'Published')),
        (DELETED, _(u'Deleted')),
    )
    
    title = models.CharField(_(u"Title"), max_length=150)
    slug = models.SlugField(_(u"Slug"), unique=True, max_length=150, db_index=True)
    author = models.ForeignKey(User, verbose_name=_(u"Author"))
    excerpt = models.TextField(_(u"Excerpt"), blank=True, db_column="exceprt")
    content = models.TextField(_(u"Content"))
    main_picture = models.ForeignKey(Picture, verbose_name=_("Picture"), blank=True, null=True)

    published_on = models.DateTimeField(_(u"Published on"), db_index=True)
    created_on = models.DateTimeField(auto_now_add=True, editable=False)
    updated_on = models.DateTimeField(auto_now=True, editable=False)
    status = models.IntegerField(_(u"Status"), choices=STATUS_CHOICES, db_index=True, default=DRAFT)
    
    selected = models.BooleanField(_(u"Selected"), default=False)
    comments_open = models.BooleanField(_(u"Are comments open?"), default=True)

    categories = models.ManyToManyField(Category, verbose_name=_(u"Categories"))

    site = models.ForeignKey(Site, verbose_name=_(u"Site"), default=settings.SITE_ID)

    # Managers
    objects = PostManager()
    on_site = CurrentSiteManager()
    availables = AvailableItemsManager()  # The Online manager.

    class Meta:
        ordering = ['-published_on']
        verbose_name = _(u"item")
        verbose_name_plural = _(u"items")

    def __unicode__(self):
        return self.title

    def natural_key(self):
        return [self.slug, self.site.id]

    def get_absolute_url(self):
        try:
            kwargs = {
                'year': self.published_on.strftime("%Y"),
                'month': self.published_on.strftime("%m"),
                'day': self.published_on.strftime("%d"),
                'slug': self.slug
            }
            return reverse('blog-item', kwargs=kwargs, urlconf=settings.ROOT_URLCONF)
        except NoReverseMatch:
            try:
                kwargs = {
                    'year': self.published_on.strftime("%Y"),
                    'month': self.published_on.strftime("%m"),
                    'slug': self.slug
                }
                return reverse('blog-item', kwargs=kwargs, urlconf=settings.ROOT_URLCONF)
            except NoReverseMatch:
                kwargs = {
                    'slug': self.slug
                }
                return reverse('blog-item', kwargs=kwargs, urlconf=settings.ROOT_URLCONF)

    def __item_cache_key(self):
        """
        Return a unique item key that can be used in order to cache it
        """
        return "blogging:post:{0}".format(self.id)

    def related_items(self):
        """
        Return items related to the current item
        """
        return Post.availables.related_items(self)



import datetime

from django.contrib.auth.models import User
from django.test import TestCase
from django.test.utils import override_settings

from blogging.models import Post


class PostUrlsTestCase(TestCase):
    def setUp(self):
        User.objects.create(username='test_user')

    @override_settings(ROOT_URLCONF='blogging.urls')
    def test_long_urls(self):
        post_1, created = Post.objects.get_or_create(
            title="post 1", slug="post-1",
            published_on=datetime.datetime(2010, 1, 1),
            author=User.objects.get(username='test_user')
        )
        self.assertEqual(post_1.get_absolute_url(), '/2010/01/01/post-1')

    @override_settings(ROOT_URLCONF='blogging.urls.short')
    def test_short_urls(self):
        post_1, created = Post.objects.get_or_create(
            title="post 1", slug="post-1",
            published_on=datetime.datetime(2010, 1, 1),
            author=User.objects.get(username='test_user')
        )
        self.assertEqual(post_1.get_absolute_url(), '/post-1')

    @override_settings(ROOT_URLCONF='blogging.urls.yearmonth')
    def test_yearmonth_urls(self):
        post_1, created = Post.objects.get_or_create(
            title="post 1", slug="post-1",
            published_on=datetime.datetime(2010, 1, 1),
            author=User.objects.get(username='test_user')
        )
        self.assertEqual(post_1.get_absolute_url(), '/2010/01/post-1')
